/*
 * Copyright (c) 2019 Nordic Semiconductor ASA
 *
 * SPDX-License-Identifier: LicenseRef-BSD-5-Clause-Nordic
 */

#include <zephyr.h>
#include <nrf_socket.h>
#include <net/socket.h>
#include <stdio.h>
#include <modem/at_cmd.h>
#include <modem/at_notif.h>
#include <errno.h>

#define AT_CMD_SIZE(x) (sizeof(x) - 1)

LOG_MODULE_REGISTER(testModule, LOG_LEVEL_DBG);
K_SEM_DEFINE(lte_ready, 0, 1);

int tcpSocket_Create(void)
{
        return socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);  
}

int tcpSocket_Close(int iSocket)
{
	if (iSocket > 0) {
		return close(iSocket);
	}

	return 0;
}

int tcpSocket_Connect(int iSocket, const char *url, u16_t port) 
{
        struct addrinfo *result;
        struct addrinfo hints = {
                .ai_family = AF_INET,
                .ai_socktype = SOCK_STREAM
        };
        
        int err = getaddrinfo(url, NULL, &hints, &result);
        if (err) {
                LOG_ERR("ERROR: getaddrinfo failed %d", err);
                return err;
        }

        if (result == NULL) {
                LOG_ERR("ERROR: Address not found\n");
                return -ENOENT;
        }

        /* IPv4 Address. */
        struct sockaddr_in server;

        server.sin_addr.s_addr = ((struct sockaddr_in *)result->ai_addr)->sin_addr.s_addr;
        server.sin_family = AF_INET;
        server.sin_port = htons(port);

        /* Free the address. */
        freeaddrinfo(result);

        return connect(iSocket, (struct sockaddr *)&server, sizeof(struct sockaddr_in));
}

static const char status1[] = "+CEREG: 1";
static const char status2[] = "+CEREG:1";
static const char status3[] = "+CEREG: 5";
static const char status4[] = "+CEREG:5";

static void wait_for_lte(void *context, const char *response)
{
	if (!memcmp(status1, response, AT_CMD_SIZE(status1)) ||
		!memcmp(status2, response, AT_CMD_SIZE(status2)) ||
		!memcmp(status3, response, AT_CMD_SIZE(status3)) ||
		!memcmp(status4, response, AT_CMD_SIZE(status4))) {
		k_sem_give(&lte_ready);
	}
}

void main(void)
{
        LOG_INF("Starting App");
        at_notif_register_handler(NULL, wait_for_lte);
        // Sleep to show sleep current before sockets
        LOG_INF("Sleeping for 30 seconds");
        k_sleep(K_SECONDS(30));                
        
        while (1)
        {
                LOG_INF("Booting Modem");
                int err = at_cmd_write("AT+CFUN=1", NULL, 0, NULL);
                if (err) {
                        return;
                }
                
                LOG_INF("Waiting for registration");
                if (at_cmd_write("AT+CEREG=2", NULL, 0, NULL) != 0) {
                        return;
                }
                
                k_sem_take(&lte_ready, K_FOREVER);
                
                LOG_INF("Opening Socket");
                int iSocket = tcpSocket_Create();
                if (iSocket < 0) {
                        return;
                }
        
                LOG_INF("Connecting Socket");
                err = tcpSocket_Connect(iSocket, "google.com", 80);
                if (err) {
                        return;
                }
        
                LOG_INF("Closing Socket");
                err = tcpSocket_Close(iSocket);
                if (err) {
                        return;
                }
                
                LOG_INF("Powering down modem");
                err = at_cmd_write("AT+CFUN=4", NULL, 0, NULL);
                if (err) {
                        return;
                }
                
                LOG_INF("Sleeping for 1 minute");
                k_sleep(K_MINUTES(1));
        }
}
